using UnityEngine;
using System.Collections;
using System.Collections.Generic;
//using System;

public class LevelGenerator : MonoBehaviour {
	
	Color32 EMPTY = new Color32(0xCC, 0xCC, 0xCC, 0xff);
	Color32 FILLED = new Color32(0, 0, 0, 0);
	Color32 START = new Color32(0, 0, 0xFF, 0);
	Color32 END = new Color32(0xFF, 0, 0, 0);
	
	public Transform prefab;
	
	public int Spacing = 64;
	
	public int LevelBlockWidth = 32;
	public int LevelBlockHeight = 32;
	
	public int MinSize=0;
	public int MaxSize=0;
	public float TurnRatio = 0.5f;
	
	List<Transform> levelBlocks = new List<Transform>();
	
	// Use this for initialization
	void Start () {
		mapData = new Color32[LevelBlockWidth * LevelBlockHeight];
		//GenerateLevel();
		
		GenerateCave(MinSize, MaxSize, TurnRatio, "", 0);
		
		GenerateLevel(false);
	}
	
	// Update is called once per frame
	void Update () {
	
		if (Input.GetKeyUp (KeyCode.G)) {
			GenerateCave(MinSize, MaxSize, TurnRatio, "", 0);
			GenerateLevel(true);
		}
	}
	
	void GenerateLevel(bool rebuild) {
		for( int i=0; i<LevelBlockWidth; i++) {
			for( int j=0; j<LevelBlockHeight; j++) {
				Color32 c = mapData[i + j*LevelBlockWidth];
				Transform obj;
				if (rebuild) {
					obj = levelBlocks[i + j*LevelBlockWidth];
				} else {
					Vector3 pos = new Vector3(i * Spacing, 0.0f, -j*Spacing);
					obj = (Transform)GameObject.Instantiate(prefab, pos, Quaternion.identity);
					levelBlocks.Add(obj);
				}
				obj.gameObject.renderer.material.color = c;
			}
		}
	}
	
	Color32[] mapData;
	int startX;
	int startY;
	int endX;
	int endY;
	int cellCount = 0;
	
	int cellx;
	int celly;
	
	public void GenerateCave(int minSize /*= 0*/, int maxSize /*= 0*/, float turnRatio /*= .5*/, string type, int iter /* = 0*/) {
		
		// Clear the Current Map
		cellCount = 1;
		//mapData.fillRect(mapData.rect, EMPTY);
		for (int i=0; i<mapData.Length; i++) {
			mapData[i] = EMPTY;
		}

		// Create the Start Cell
		startX = LevelBlockWidth / 2;
		startY = LevelBlockHeight / 2;
		mapData[startX + LevelBlockWidth * startY] = FILLED;
                       
		// Create the X/Y List with the Start Cell
		List<int> xList = new List<int>(new int[]{startX});
		List<int> yList = new List<int>(new int[]{startY});
                       
		// Other various vars
		int openPaths 	= 1;
		List<int> orderList = new List<int>(new int[]{0, 1, 2, 3});
		orderList.Shuffle();
		
		int[] xoffset = new int[]{1, 0, -1, 0};
		int[] yoffset = new int[]{0, -1, 0, 1};
		int adjacentNum = 2;
		int cellx = 0;
		int celly = 0;
                               
		// For each Cell on the List
		for (int p = 0; p < cellCount; p++) {
			// Get the Cell we are working with
			cellx = xList.Pop();
			celly = yList.Pop();
                               
			// Decide how many adjacent Cells to create around this Cell
			adjacentNum = AdjacentCells(openPaths, type);
                               
			// Shuffle the Order List
			if (Random.Range(0.0f,1.0f) < turnRatio)
				orderList.Shuffle();
			
			// For each Adjacent Position (dealt with in the order as defined by orderList)
			if (adjacentNum > 0) {
				for (int a = 0; a < 4; a++) {
					int adjx = cellx + xoffset[orderList[a]];
					int adjy = celly + yoffset[orderList[a]];
                                                       
					if (adjx >= 0 && adjx < LevelBlockWidth && adjy >= 0 && adjy < LevelBlockHeight) {
						if (mapData[adjx + adjy * LevelBlockWidth].Equals(EMPTY)) {
							// Add and Fill the adjacent Cell
							xList.Push(adjx);
							yList.Push(adjy);
							mapData[adjx + adjy * LevelBlockWidth] = FILLED;
							openPaths++;
							cellCount++;
							adjacentNum--;
                                                               
							// Break when we can't create any more adjacent Cells
							if (adjacentNum == 0) break;
						}
					}
					if (cellCount == maxSize) break;
				}
				if (cellCount == maxSize) break;
			}
			openPaths --;
		}
                       
		// Use recursive check to minimize the map size
		if (cellCount - iter < minSize) {
			GenerateCave(minSize, maxSize, turnRatio, type, iter + 10);
		} else {
			endX = cellx;
			endY = celly;
			mapData[startX + startY * LevelBlockWidth] = START;
			mapData[endX + endY * LevelBlockWidth] = END;
		}
	}
	
	
 	// CHOOSE HOW MANY ADJACENT PATHS
	public int AdjacentCells(int openPaths, string type) {
		switch(type) {
			case "THIN": {
				if (openPaths > 1) {
					// This is one of multiple open paths
					switch (Random.Range(0,7)) {
						case (0): {
							return 0; 
						}
						case (1): {
							return 2;
						}
					
						default: return 1;
					}
				} else {
					// This is the last open path
					return 1;
				}
			}
                               
			case "WIDE": {
				if (openPaths > 1) {
					// This is one of multiple open paths
					switch(Random.Range (0,4)) {
						case 0: return 1;
						case 1: return 2;
						case 2: return 3;
						case 3: return 4;
						default: return 1;
					}
				} else {
					// This is the last open path
					switch(Random.Range(0,3)) {
						case 0: return 1;
						default: return 2;
					}
				}
			}
                               
			default: {
				if (openPaths > 1) {		
					// This is one of multiple open paths
					switch(Random.Range(0,4)) {
						case 0: return 0;
						case 1: return 1;
						case 2: return 1;
						case 3: return 2;
						default: return 1;
					}
				} else {
					// This is the last open path
					switch(Random.Range (0,4)) {
						case 0: return 1;
						case 1: return 1;
						case 2: return 1;
						case 3: return 2;
						default: return 1;
					}
				}
				break;
			}
		}
		return 0;
	}
}
