// unlit, vertex colour, alpha blended
// cull off

Shader "MyLighting/MySprite" 
{
	Properties 
	{
		_MainTex ("Base (RGB) Trans (A)", 2D) = "white" {}
		_LightTex ("Light", 2D) = "grey" {}
	}
	
	SubShader
	{
		Tags {"Queue"="Transparent" "IgnoreProjector"="True" "RenderType"="Transparent"}
		ZWrite Off Lighting Off Cull Off Fog { Mode Off } Blend SrcAlpha OneMinusSrcAlpha
		LOD 110
		
		Pass 
		{
			CGPROGRAM
			#pragma vertex vert_vct
			#pragma fragment frag_mult 
			#pragma fragmentoption ARB_precision_hint_fastest
			#include "UnityCG.cginc"

			sampler2D _MainTex;
			sampler2D _LightTex;
			float4 _MainTex_ST;
			float4 _LightTex_ST;

			struct vin_vct 
			{
				float4 vertex : POSITION;
				float4 color : COLOR;
				float2 texcoord : TEXCOORD0;
			};

			struct v2f_vct
			{
				float4 vertex : POSITION;
				fixed4 color : COLOR;
				half2 texcoord : TEXCOORD0;
				float4 projPos : TEXCOORD1;
			};

			v2f_vct vert_vct(vin_vct v)
			{
				v2f_vct o;
				o.vertex = mul(UNITY_MATRIX_MVP, v.vertex);
				o.projPos = ComputeScreenPos (o.vertex);
				o.color = v.color;
				o.texcoord = v.texcoord;
				return o;
			}

			float4 frag_mult(v2f_vct i) : COLOR
			{
				float4 refractionUv = i.projPos;
				float3 light  = tex2Dproj(_LightTex, refractionUv);

				float4 cc = tex2D(_MainTex, i.texcoord);
				
				//fixed4 cc = tex2D(_MainTex, i.texcoord) * i.color;
				//fixed4 col = tex2D(_MainTex, i.texcoord) * i.color;
				//return col;

				//return fixed4(cc.xyz * light * 2.0f * i.color, cc.w);
				//return fixed4(light, cc.w);
				//return cc;
				return light;
			}
			
			ENDCG
		} 
	}
 
	SubShader
	{
		Tags {"Queue"="Transparent" "IgnoreProjector"="True" "RenderType"="Transparent"}
		ZWrite Off Blend SrcAlpha OneMinusSrcAlpha Cull Off Fog { Mode Off }
		LOD 100

		BindChannels 
		{
			Bind "Vertex", vertex
			Bind "TexCoord", texcoord
			Bind "Color", color
		}

		Pass 
		{
			Lighting Off
			SetTexture [_MainTex] { combine texture * primary } 
		}
	}
}
